@@ -62,8 +62,6 @@ def tds_stream(alias):
                 current_app.redis.hset(f"tds_uniq_{alias}", uniq_key, ts)
             logger.info(f'uniq: {is_uniq}')
 
-            logger.info(f'referrer: {request.referrer}')
-
             # '2001:db8::' or 
             ip_obj = ipaddress.ip_address(current_user.ip)
             is_ipv6 = ip_obj.version == 6
@@ -112,6 +110,13 @@ def tds_stream(alias):
                             url = url.replace(snip, v)
 
                 # send task to create activity
+                logger.info(f'Referrer: {request.referrer}')
+
+                referer = None
+                if r := request.referrer:
+                    uri_ref = urlparse(r)
+                    referer = uri_ref.path
+
                 _doc = {
                     'createdon': datetime.utcnow(),
                     'stream': _s['id'],
@@ -128,6 +133,7 @@ def tds_stream(alias):
                     'action': _s['action'],
                     'url': url if _s['action'] in ['http', 'campaign', 'js'] else None,
                     'subid': subid,
+                    'referrer': referer,
                     'fingerprint': hashed_uniq_key
                 }
                 tdscampaign_hit.apply_async(args=[_doc])
@@ -246,7 +252,11 @@ def tds_postback(cid):
     logger.info(f'Postback args: {request.args}')
     logger.info(f'Postback data: {request.data}')
     logger.info(f'Postback form: {request.form}')
-    logger.info(f'Postback json: {request.get_json()}')
+
+    req_payload = {}
+    if request.content_type == 'application/json':
+        req_payload = request.json
+        logger.info(f'Postback json: {req_payload}')
 
     campaigns, found = TdsCampaign.get(_id=cid)
     if found == 1:
@@ -257,7 +267,7 @@ def tds_postback(cid):
             if uri_campaign.hostname == uri.hostname:
                 pid = TdsPostback.get_urlsafe_string(12)
 
-                payload = request.form or request.get_json() or {}
+                payload = request.form or req_payload or {}
                 logger.info(f'Postback payload: {payload}')
                 
                 _doc = {
@@ -273,6 +283,7 @@ def tds_postback(cid):
                     'payload': dumps(payload),
                 }
 
+                logger.info(f'Postback doc: {_doc}')
                 # process by campaign.postback_processor or default args (action)
 
                 _id = TdsPostback.generate_id(_doc['createdon'], campaign._id, pid)
